@@ -4,8 +4,8 @@ package Proc::Forking;
 # Fork package
 # Gnu GPL2 license
 #
-# $Id: Forking.pm,v 1.14 2004/11/29 13:42:53 fabrice Exp $
-# $Revision: 1.14 $
+# $Id: Forking.pm,v 1.16 2004/11/29 19:39:30 fabrice Exp $
+# $Revision: 1.16 $
 #
 # Fabrice Dulaunoy <fabrice@dulaunoy.com>
 ###########################################################
@@ -20,19 +20,19 @@ use Cwd;
 use Sys::Load qw/getload/;
 use vars qw($VERSION );
 
-my $CVS_version = '$Revision: 1.14 $';
+my $CVS_version = '$Revision: 1.16 $';
 $CVS_version =~ s/\$//g;
-my $CVS_date = '$Date: 2004/11/29 13:42:53 $';
+my $CVS_date = '$Date: 2004/11/29 19:39:30 $';
 my $REVISION = "version $CVS_version created $CVS_date";
 $CVS_version =~ s/Revision: //g;
 my $VERSIONA = $';
 $VERSIONA =~ s/ //g;
-$VERSION = do { my @rev = (q$Revision: 1.14 $ =~ /\d+/g); sprintf "%d."."%d" x $#rev, @rev };
+$VERSION = do { my @rev = ( q$Revision: 1.16 $ =~ /\d+/g ); sprintf "%d." . "%d" x $#rev, @rev };
 $REVISION =~ s/\$Date: //g;
 my $DAEMON_PID;
 $SIG{ CHLD } = \&garbage_child;
-$SIG{ INT } =  $SIG{ TERM } =  $SIG{ HUP } =
-  sub { killall_childs(); unlink $DAEMON_PID;};
+$SIG{ INT } = $SIG{ TERM } = $SIG{ HUP } =
+  sub { killall_childs(); unlink $DAEMON_PID; };
 
 my %PID;
 my %NAME;
@@ -77,7 +77,7 @@ sub daemonize
         return ( $CODE[13][0], 0, $CODE[13][1] );
     }
     exit 0 if $child;    # parent dies;
-    my $ret = create_pid_file( $pid_file, $$ );
+    my $ret  = create_pid_file( $pid_file, $$ );
     my $luid = -1;
     my $lgid = -1;
     if ( $uid ne '' )
@@ -93,7 +93,7 @@ sub daemonize
     {
         local ( $>, $< ) = ( $<, $> );
         my $cwd = $home;
-        chdir( $cwd ) || return ( $CODE[11][0], 0, $CODE[11][1] );
+        chdir( $cwd )  || return ( $CODE[11][0], 0, $CODE[11][1] );
         chroot( $cwd ) || return ( $CODE[12][0], 0, $CODE[12][1] );
         $< = $>;
     }
@@ -132,8 +132,8 @@ sub new
         _gid       => $_[8],
         _max_child => $_[9],
         _max_load  => $_[10],
-	_pids      => $_[11],
-	_names     => $_[12],
+        _pids      => $_[11],
+        _names     => $_[12],
     }, $class;
 
 }
@@ -174,7 +174,7 @@ sub fork_child
     if ( exists( $param{ max_child } ) )
     {
         $self->{ _max_child } = $param{ max_child };
-        if ( $self->{ _max_child } <= ( keys  %{$self->{_pids} }) )
+        if ( $self->{ _max_child } <= ( keys %{ $self->{ _pids } } ) )
         {
             return ( $CODE[5][0], 0, $CODE[5][1] );
         }
@@ -198,49 +198,54 @@ sub fork_child
             {
                 $pid_file = $self->{ _pid_file };
                 $pid_file =~ s/##/$pid/g;
-            }	
-		 if ( !defined( $self->{ _pids }{ $pid } ) )
-		{	
-		$self->{ _pids }{ $pid }{ name } = $exp_name;
-		 if ( defined( $self->{ _pid_file } ) )
+            }
+            if ( !defined( $self->{ _pids }{ $pid } ) )
+            {
+                $self->{ _pids }{ $pid }{ name } = $exp_name;
+                if ( defined( $self->{ _pid_file } ) )
                 {
-		    $self->{ _pids }{ $pid }{ pid_file } = $pid_file;
-		    $PID{ pid_file } = $pid_file;
+                    $self->{ _pids }{ $pid }{ pid_file } = $pid_file;
+                    $PID{ pid_file } = $pid_file;
                 }
-		 if ( defined( $self->{ _home } ) )
+                if ( defined( $self->{ _home } ) )
                 {
-		    $self->{ _pids }{ $pid }{ home } = $self->{ _home };
-		    $PID{ home } = $self->{ _home };
+                    $self->{ _pids }{ $pid }{ home } = $self->{ _home };
+                    $PID{ home } = $self->{ _home };
                 }
             }
             else
             {
                 return ( $CODE[9][0], $self->{ _pid }, $CODE[9][1] );
-            }		
-            if ( !defined( $self->{ _names }{$exp_name} ) ){		
-               $self->{ _names }{ $exp_name }{ pid } = $pid;
-		    $NAME{ $exp_name }{ pid } = $pid;
-		 if ( defined( $self->{ _pid_file } ) )
+            }
+            if ( !defined( $self->{ _names }{ $exp_name } ) )
+            {
+                $self->{ _names }{ $exp_name }{ pid } = $pid;
+                $NAME{ $exp_name }{ pid } = $pid;
+                if ( defined( $self->{ _pid_file } ) )
                 {
-		    $self->{ _names }{ $exp_name }{ pid_file } = $pid_file;
-		    $NAME{ $exp_name }{ pid_file } = $pid_file;
+                    $self->{ _names }{ $exp_name }{ pid_file } = $pid_file;
+                    $NAME{ $exp_name }{ pid_file } = $pid_file;
                 }
                 if ( defined( $self->{ _home } ) )
                 {
-		    $self->{ _names }{ $exp_name }{ home } = $self->{ _home }; 
-		    $NAME{ $exp_name }{ home } =  $self->{ _home };
-                };
+                    $self->{ _names }{ $exp_name }{ home } = $self->{ _home };
+                    $NAME{ $exp_name }{ home } = $self->{ _home };
+                }
             }
             else
             {
-                return ( $CODE[10][0], $self->{ _pid }, ( $self->{ _name }.$CODE[10][1] ));
+                return (
+                    $CODE[10][0],
+                    $self->{ _pid },
+                    ( $self->{ _name } . $CODE[10][1] )
+                );
             }
             return ( $CODE[0][0], $self->{ _pid }, $CODE[0][1] );
         }
         elsif ( defined $pid )
         {
 ## in  child
-        $SIG{HUP} = $SIG{INT} = $SIG{CHLD} = $SIG{TERM} = 'DEFAULT';
+            $SIG{ HUP } = $SIG{ INT } = $SIG{ CHLD } = $SIG{ TERM } = 'DEFAULT';
             if ( defined( $self->{ _name } ) )
             {
                 my $exp_name = $self->{ _name };
@@ -253,7 +258,7 @@ sub fork_child
             {
                 local ( $>, $< ) = ( $<, $> );
                 my $cwd = $self->{ _home };
-                chdir( $cwd ) || return ( $CODE[11][0], 0, $CODE[11][1] );
+                chdir( $cwd )  || return ( $CODE[11][0], 0, $CODE[11][1] );
                 chroot( $cwd ) || return ( $CODE[12][0], 0, $CODE[12][1] );
                 $< = $>;
             }
@@ -275,7 +280,7 @@ sub fork_child
                 $pid_file = $self->{ _pid_folder } . $pid_file;
             }
             $ret = create_pid_file( $pid_file, $$ );
-            $self->{ _function } ( $self->{ _args } );
+            $self->{ _function }( $self->{ _args } );
             if ( defined $self->{ _pid_file } )
             {
                 my $pid_file = $self->{ _pid_file };
@@ -310,54 +315,59 @@ sub kill_child
     my $pid    = shift;
     my $signal = shift || 15;
     kill $signal => $pid;
-   
-    my $name = $self->{ $pid }{ name };
-    if ( defined $self->{ _pids }{ $pid }{ pid_file } )
+    my $state = kill 0 => $pid;
+    if ( !$state )
     {
-        my $pid_file = $self->{_pids}{ $pid }{ pid_file };
-        $pid_file =~ s/##/$pid/g;
-        delete $self->{_pids}{ $pid }{ pid_file };
-        delete $self->{_names}{ $name }{ pid_file };
-        if ( defined $self->{ _pid_file }{ _home } )
+        my $name = $self->{ $pid }{ name };
+        if ( defined $self->{ _pids }{ $pid }{ pid_file } )
         {
-            $pid_file = $self->{ _pid_file }{ _home } . $pid_file;
-        }
-        if ( -e $pid_file )
-        {
-            delete_pid_file( $pid_file );
+            my $pid_file = $self->{ _pids }{ $pid }{ pid_file };
+            $pid_file =~ s/##/$pid/g;
+            delete $self->{ _pids }{ $pid }{ pid_file };
+            delete $self->{ _names }{ $name }{ pid_file };
+            if ( defined $self->{ _pid_file }{ _home } )
+            {
+                $pid_file = $self->{ _pid_file }{ _home } . $pid_file;
+            }
+            if ( -e $pid_file )
+            {
+                delete_pid_file( $pid_file );
+            }
         }
+        delete $self->{ _pids }{ $pid }{ name };
+        delete $self->{ _pids }{ $pid };
+
+        delete $self->{ _names }{ $name }{ pid };
+        delete $self->{ _names }{ $name };
+
+        delete $PID{ $pid }{ name };
+        delete $PID{ $pid };
+
+        delete $NAME{ $name }{ pid };
+        delete $NAME{ $name };
     }
-    delete $self->{_pids}{ $pid }{ name };
-    delete $self->{_pids}{ $pid };
-
-    delete $self->{_names}{ $name }{ pid };
-    delete $self->{_names}{ $name };
-    
-    delete $PID{ $pid }{ name };
-    delete $PID{ $pid };
-
-    delete $NAME{ $name }{ pid };
-    delete $NAME{ $name };
+##	$self->clean_childs();;
 }
 
 sub killall_childs
 {
-    my $self = shift;
+    my $self   = shift;
     my $signal = shift || 15;
-    my $pids = $self->{_pids};
-    my %pids = %{$pids};
-    foreach (  keys  %pids)
+    my $pids   = $self->{ _pids };
+    my %pids   = %{ $pids };
+    foreach ( keys %pids )
     {
-	kill_child( $self ,$_,$signal);	
+        kill $signal => $_;
     }
-    $SIG{ INT } = $SIG{ TERM } = $SIG{ HUP } = 'DEFAULT';    
-}
+    $self->clean_childs();
 
+#    $SIG{ INT } = $SIG{ TERM } = $SIG{ HUP } = 'DEFAULT';
+}
 
 sub list_pids
 {
-    my $self = shift; 
-    return $self->{ _pids };;
+    my $self = shift;
+    return $self->{ _pids };
 }
 
 sub list_names
@@ -369,7 +379,7 @@ sub list_names
 sub pid_nbr
 {
     my $self = shift;
-    return ( scalar( keys  %{$self->{ _pids }} ) );
+    return ( scalar( keys %{ $self->{ _pids } } ) );
 }
 
 sub clean_childs
@@ -378,21 +388,21 @@ sub clean_childs
     my @pid_remove_list;
     my @name_remove_list;
 
-    foreach my $child ( keys %{ $self->{ _pids }} )
+    foreach my $child ( keys %{ $self->{ _pids } } )
     {
 
         my $state = kill 0 => $child;
         if ( !$state )
         {
-            my $name =  $self->{ _pids }{ $child }{ name };
-            if ( defined  $self->{_pids }{ $child }{ pid_file } )
-            { 	    
-                my $pid_file =  $self->{ _pids }{ $child }{ pid_file };
+            my $name = $self->{ _pids }{ $child }{ name };
+            if ( defined $self->{ _pids }{ $child }{ pid_file } )
+            {
+                my $pid_file = $self->{ _pids }{ $child }{ pid_file };
                 delete $self->{ _pids }{ $child }{ pid_file };
-                delete  $self->{ _names }{ $name }{ pid_file };
-                if ( defined  $self->{ _pids }{ $child }{ home } )
+                delete $self->{ _names }{ $name }{ pid_file };
+                if ( defined $self->{ _pids }{ $child }{ home } )
                 {
-                    $pid_file =  $self->{ _pids }{ $child }{ home } . $pid_file;
+                    $pid_file = $self->{ _pids }{ $child }{ home } . $pid_file;
                 }
 
                 if ( -e $pid_file )
@@ -400,9 +410,9 @@ sub clean_childs
                     delete_pid_file( $pid_file );
                 }
             }
-            delete  $self->{ _pids }{ $child }{ name };
-            delete  $self->{ _pids }{ $child };
-            delete  $self->{ _names }{ $name }{ pid };
+            delete $self->{ _pids }{ $child }{ name };
+            delete $self->{ _pids }{ $child };
+            delete $self->{ _names }{ $name }{ pid };
             delete $self->{ _names }{ $name };
             push @pid_remove_list,  $child;
             push @name_remove_list, $name;
@@ -416,11 +426,11 @@ sub test_pid
     my $self  = shift;
     my $child = shift;
     my $state;
-    if ( defined  $self->{_pids}{ $child } )
+    if ( defined $self->{ _pids }{ $child } )
     {
         $state = kill 0 => $child;
     }
-    return ( $state, (  $self->{_pids}{ $child }{ name } ) );
+    return ( $state, ( $self->{ _pids }{ $child }{ name } ) );
 }
 
 sub test_name
@@ -428,11 +438,11 @@ sub test_name
     my $self = shift;
     my $name = shift;
     my $state;
-    if ( defined(  $self->{_names}{ $name }{ pid } ) )
+    if ( defined( $self->{ _names }{ $name }{ pid } ) )
     {
-        $state = kill 0 => ( $self->{_names}{ $name }{ pid } );
+        $state = kill 0 => ( $self->{ _names }{ $name }{ pid } );
     }
-    return ( $state, ( $self->{_names}{ $name }{ pid } ) );
+    return ( $state, ( $self->{ _names }{ $name }{ pid } ) );
 }
 
 sub version
@@ -460,6 +470,7 @@ sub create_pid_file
     }
     if ( -e $file )
     {
+
 # pid file already exists
         my $fh      = IO::File->new( $file );
         my $pid_num = <$fh>;
@@ -474,7 +485,7 @@ sub create_pid_file
         }
     }
     my $fh = IO::File->new( $file, O_WRONLY | O_CREAT | O_EXCL, 0644 );
-    if ( !$fh ) { return ( $CODE[2][0], $pid_num, $CODE[2][1] );; }
+    if ( !$fh ) { return ( $CODE[2][0], $pid_num, $CODE[2][1] ); }
     print $fh $pid_num;
     return ( $CODE[0][0], $pid_num, $CODE[0][1] );
 }
@@ -495,16 +506,16 @@ sub garbage_child
 {
     while ( ( my $child = waitpid( -1, WNOHANG ) ) > 0 )
     {
-        my $name =  $PID{ $child }{ name };
-        if ( defined  $PID{ $child }{ pid_file } )
+        my $name = $PID{ $child }{ name };
+        if ( defined $PID{ $child }{ pid_file } )
         {
-            my $pid_file =  $PID{ $child }{ pid_file };
+            my $pid_file = $PID{ $child }{ pid_file };
             $pid_file =~ s/##/$child/g;
-            delete  $PID{ $child }{ pid_file };
-            delete  $NAME{ $name }{ pid_file };
-            if ( defined  $PID{ $child }{ home } )
+            delete $PID{ $child }{ pid_file };
+            delete $NAME{ $name }{ pid_file };
+            if ( defined $PID{ $child }{ home } )
             {
-                $pid_file =  $PID{ $child }{ home } . $pid_file;
+                $pid_file = $PID{ $child }{ home } . $pid_file;
             }
 
             if ( -e $pid_file )
@@ -513,10 +524,10 @@ sub garbage_child
             }
         }
 
-        delete  $PID{ $child }{ name };
-        delete  $PID{ $child };
+        delete $PID{ $child }{ name };
+        delete $PID{ $child };
 
-        delete  $NAME{ $name }{ pid };
+        delete $NAME{ $name }{ pid };
         delete $NAME{ $name };
     }
     $SIG{ CHLD } = \&garbage_child;
